/*
 * thread.c
 *
 * Dominik Madon, LSL/DI/EPFL, 1998
 *
 * This code interfaces assembler instructions to send and
 * receive messages between contexts in the ssmt.
 *
 * This code is part of libssmt.a
 *
 */

#include "libssmt.h"


int executeThread(
      SS_CONTEXT_TYPE context,
      unsigned int thread_id,
      SS_WORD_TYPE *stack,
      unsigned int stack_sz,
      SS_WORD_TYPE (*func)())
{
  SS_CONTEXT_TYPE me;
  unsigned int reg;
  SS_WORD_TYPE i;

  me = contextID();
  i = thread_id;        /* suppress the warning but not needed */

  for(reg = 0; reg < SS_REG_MAX; reg++) {
    asm("rcxt %0, %1, %2" : "=r" (i) : "r" (me), "r" (reg));
    asm("wcxt %0, %1, %2" :: "r" (i), "r" (context), "r" (reg));
  }
  reg = SS_REG_I29; i = (SS_WORD_TYPE) stack + stack_sz;
  asm("wcxt %0, %1, %2" :: "r" (i), "r" (context), "r" (reg));
  reg = SS_REG_I31; i = (SS_WORD_TYPE) context;
  asm("wcxt %0, %1, %2" :: "r" (i), "r" (context), "r" (reg));
  reg = SS_REG_PC; i = (SS_WORD_TYPE) func;
  asm("wcxt %0, %1, %2" :: "r" (i), "r" (context), "r" (reg));
  reg = SS_MMU_CONFIG;
  asm("wcxt %0, %1, %2" :: "r" (me), "r" (context), "r" (reg));

  resumeContext(context);

  return 0;
}





void returnThread(SS_WORD_TYPE value)
{
  SS_CONTEXT_TYPE me;
  unsigned int reg;

  me = contextID();
  reg = SS_REG_I30;
  asm("wcxt %0, %1, %2" :: "r" (value), "r" (me), "r" (reg));
  reg = SS_REG_I31;
  asm("rcxt %0, %1, %2" : "=r" (value) : "r" (me), "r" (reg));
  sendToContext((SS_WORD_TYPE) 0, value);
  suspendContext(me);
}




SS_WORD_TYPE waitThread(SS_CONTEXT_TYPE context)
{
  SS_WORD_TYPE value;
  unsigned int reg;
  
  receiveFromContext(&value, context);
  reg = SS_REG_I30;
  asm("rcxt %0, %1, %2" : "=r" (value) : "r" (context), "r" (reg));
  return value;
}
