/*
 * regs.c - architected registers state routines
 *
 * This file is a part of the SimpleScalar tool suite written by
 * Todd M. Austin as a part of the Multiscalar Research Project.
 *  
 * The tool suite is currently maintained by Doug Burger and Todd M. Austin.
 * 
 * Copyright (C) 1994, 1995, 1996, 1997 by Todd M. Austin
 *
 * This source file is distributed "as is" in the hope that it will be
 * useful.  The tool set comes with no warranty, and no author or
 * distributor accepts any responsibility for the consequences of its
 * use. 
 * 
 * Everyone is granted permission to copy, modify and redistribute
 * this tool set under the following conditions:
 * 
 *    This source code is distributed for non-commercial use only. 
 *    Please contact the maintainer for restrictions applying to 
 *    commercial use.
 *
 *    Permission is granted to anyone to make or distribute copies
 *    of this source code, either as received or modified, in any
 *    medium, provided that all copyright notices, permission and
 *    nonwarranty notices are preserved, and that the distributor
 *    grants the recipient permission for further redistribution as
 *    permitted by this document.
 *
 *    Permission is granted to distribute this file in compiled
 *    or executable form under the same conditions that apply for
 *    source code, provided that either:
 *
 *    A. it is accompanied by the corresponding machine-readable
 *       source code,
 *    B. it is accompanied by a written offer, with no time limit,
 *       to give anyone a machine-readable copy of the corresponding
 *       source code in return for reimbursement of the cost of
 *       distribution.  This written offer must permit verbatim
 *       duplication by anyone, or
 *    C. it is distributed by someone who received only the
 *       executable form, and is accompanied by a copy of the
 *       written offer of source code that they received concurrently.
 *
 * In other words, you are welcome to use, share and improve this
 * source file.  You are forbidden to forbid anyone else to use, share
 * and improve what you give them.
 *
 * INTERNET: dburger@cs.wisc.edu
 * US Mail:  1210 W. Dayton Street, Madison, WI 53706
 *
 * $Id: regs.c,v 1.6 1999/01/26 07:15:03 madon Exp $
 *
 * $Log: regs.c,v $
 * Revision 1.6  1999/01/26 07:15:03  madon
 * *** empty log message ***
 *
 * Revision 1.5  1998/08/31 09:17:26  madon
 * Synchronous messages added
 *
 * Revision 1.4  1998/07/17 11:06:36  madon
 * *** empty log message ***
 *
 * Revision 1.3  1998/02/27 16:27:11  madon
 * *** empty log message ***
 *
 * Revision 1.2  1998/01/28 12:10:42  madon
 * This release seems to have multiple registers sets supported
 * when juste one thread is running.
 *
 * Revision 1.1.1.1  1997/11/13 16:21:03  madon
 *
 * Revision 1.4  1997/03/11  01:19:28  taustin
 * updated copyright
 * long/int tweaks made for ALPHA target support
 *
 * Revision 1.3  1997/01/06  16:02:36  taustin
 * comments updated
 *
 * Revision 1.1  1996/12/05  18:52:32  taustin
 * Initial revision
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "misc.h"
#include "ss.h"
#include "loader.h"
#include "regs.h"
#include "context.h"

/* (signed) integer register file */
SS_WORD_TYPE regs_R[SS_NUM_CONTEXT][SS_NUM_REGS];

/* floating point register file */
union regs_FP regs_F;

/* (signed) hi register, holds mult/div results */
SS_WORD_TYPE regs_HI[SS_NUM_CONTEXT];
/* (signed) lo register, holds mult/div results */
SS_WORD_TYPE regs_LO[SS_NUM_CONTEXT];

/* floating point condition codes */
int regs_FCC[SS_NUM_CONTEXT];

/* program counter */
SS_ADDR_TYPE regs_PC[SS_NUM_CONTEXT];

/* Active context register */
SS_WORD_TYPE regs_ACXT;

/* Context control register */
SS_WORD_TYPE regs_CXTCTRL[SS_NUM_CONTEXT];

/* initialize architected register state */
void
regs_init(void)
{
  int i;
  SS_CONTEXT_TYPE context;

  for(context = 0; context < SS_NUM_CONTEXT; context++) {
    for(i=0; i<SS_NUM_REGS; i++)
      regs_F.l[context][i] = regs_R[context][i] = 0;
    regs_HI[context] = 0;
    regs_LO[context] = 0;
    regs_FCC[context] = 0;
    regs_CXTCTRL[context] = 0x00001f00 | context;

    /* set up initial register state */
    regs_R[context][SS_STACK_REGNO] = ld_environ_base[context];
    regs_PC[context] = ld_prog_entry[context];

  } /* for */
  regs_ACXT = 0; /* All contexts inactive */
}

/* dump all architected register state values to output stream STREAM */
void
regs_dump(FILE *stream)		/* output stream */
{
  int i;
  SS_CONTEXT_TYPE context;

  /* stderr is the default output stream */
  if (!stream)
    stream = stderr;

  /* dump processor register state */
  fprintf(stream, "Processor state:\n");
  for(context = 0; context < context_num; context++) {
    fprintf(stream, "    PC[%02d]: 0x%08x\n", context, regs_PC[context]);
    fprintf(stream, "    CXTCTRL: 0x%08x\n", regs_CXTCTRL[context]);
    for (i=0; i<SS_NUM_REGS; i += 2) {
      fprintf(stream, "    R[%02d][%2d]: %12d/0x%08x",
	      context, i, regs_R[context][i], regs_R[context][i]);
      fprintf(stream, "  R[%02d][%2d]: %12d/0x%08x\n",
	      context, i+1, regs_R[context][i+1], regs_R[context][i+1]);
    } /* for */
    fprintf(stream, "    HI:          %10d/0x%08x  LO:          %10d/0x%08x\n",
            regs_HI[context], regs_HI[context], regs_LO[context], regs_LO[context]);
    for (i=0; i<SS_NUM_REGS; i += 2) {
      fprintf(stream, "    F[%02d][%2d]: %12d/0x%08x",
              context, i, regs_F.l[context][i], regs_F.l[context][i]);
      fprintf(stream, "  F[%02d][%2d]: %12d/0x%08x\n",
              context, i+1, regs_F.l[context][i+1], regs_F.l[context][i+1]);
    } /* for */
    fprintf(stream, "    FCC:                0x%08x\n", regs_FCC[context]);
  } /* for */

}
