/*
 * bench-call-1.c
 *
 * Dominik Madon, LSL/DI/EPFL, 1998, 1999
 *
 * The goal of this executable is to time Peterson's
 * software locks.
 *
 * To use the program compile once with the context call
 * then once with the function call
 * 
 */

#include <stdio.h>
#include <string.h>
#include "ssmt-3.h"


#define EXEC_NAME               "bench-call-5"

#define FALSE                   0
#define TRUE                    1

#define SEED                    0x9a654321
#define ELEM                    4096




static unsigned int paramA[2];
static unsigned int paramB[2];
static int          *stack;
static int          alength;


unsigned int next_random(
  unsigned int *number)
{
  unsigned int newbit;

  newbit = ((0x80000000 & *number) >> 31) ^ ((0x10000000 & *number) >> 28);
  *number = (*number << 1) | newbit;

  return(*number);
} /* next_random */





void init_table(
  unsigned int *table,
  unsigned int start,
  unsigned int end)
{
  unsigned int n = SEED, i;

  for(i = start; i <= end; i++)
    table[i] = 0x3ff & next_random(&n);
} /* init_table */





int asort(unsigned int me, int * param)
{
  int i,j;
  unsigned int len, temp;
  unsigned int *table;

  table = (int *) param[0];
  len = param[1];

  for(i = 0; i < len; i++) {
    for(j = i + 1; j < len; j++) {
      if (table[i] > table[j]) {
        temp = table[i];
        table[i] = table[j];
        table[j] = temp;
      } /* if */
    } /* for */
  } /* for */

  return(0);
} /* thread_func_direct */






void qsort(
  unsigned int *table,
  int start, 
  int end)
{
  int i, j;
  unsigned int t, pivot;

  i = start - 1;
  j = end + 1;

  pivot = table[start];

  if (start < end) {
    
    while (i < j) {
      do i++; while (table[i] < pivot);
      do j--; while (table[j] > pivot);
      if (i < j) {
        t = table[i];
        table[i] = table[j];
        table[j] = t;
      } /* if */
    } /* while */

    t = j - start + 1;
    i = end - j;
    if ((t + i) < alength) {
      paramA[0] = (unsigned int) &(table[start]);
      paramA[1] = t;
      paramB[0] = (unsigned int) &(table[j + 1]);
      paramB[1] = i;
      if (t < i) {
/*          fprintf(stderr, "[%d] asort thr A/func B called\n", 0); */
        createThread(1, 1, stack, &asort, paramA);
/*          asort(1, (int *) paramA); */
        asort(1, (int *) paramB);      
        waitThread(1);
      } else {
/*          fprintf(stderr, "[%d] asort thr B/func A called\n", 0); */
        createThread(1, 1, stack, &asort, paramB);
/*          asort(1, (int *) paramB); */
        asort(1, (int *) paramA);      
        waitThread(1);
      }
      return;
    }

    qsort(table, start, j);
    qsort(table, j + 1, end);
  } /* if */

} /* qsort */





void main(int argc, char **argv)
{
  unsigned int me = 0;          /* context id for myself */
  unsigned int i,j;
  unsigned int *table;

  fprintf(stderr, "[%d] %s begin\n", me, EXEC_NAME);

  if (argc > 1) {
    fprintf(stderr, "[%d] reading parameter\n", me);    
    i = 0;
    alength = 0;
    while (argv[1][i]) {
      alength = (alength * 10) + (argv[1][i++] - '0');
    }
  } else {
    fprintf(stderr, "[%d] Missing positive parameter\n", me);    
  }

  fprintf(stderr, "[%d] initializing stack and arrays\n", me);
  stack = (int *) malloc(8*1024*32*sizeof(int));
  table = (unsigned int *) malloc((ELEM + 2)*sizeof(unsigned int));
  stack = stack + 8*1024;

  loadMMUconfig(0, 1);

  init_table(table, 1, ELEM);
  table[0] = 0;
  table[ELEM+1] = 0xfff;

  fprintf(stderr, "[%d] sorting...\n", me);

  qsort(table, 1, ELEM);

  for(i = 1; i <= (ELEM / 4); i++)
    fprintf(stderr, "[%d] %04d, %04d, %04d, %04d\n",
            me, table[i], table[i + (ELEM / 4)],
            table[i + 2 * (ELEM / 4)], table[i + 3 * (ELEM / 4)]);

  free(stack);
  free(table);

  fprintf(stderr, "[%d] %s end\n", me, EXEC_NAME);

}


