/*
 * test-interrupt-2.c
 *
 * Dominik Madon, LSL/DI/EPFL, 1998, 1999, 2000
 *
 * The goal of this executable is to check out reaction to
 * interrupts by a context when a context is running in the
 * background.
 *
 *
 */

#include <stdio.h>
#include <string.h>
#include "ssmt-3.h"


#define EXEC_NAME               "bench-interrupt-2"

#define FALSE                   0
#define TRUE                    1

#define SEED                    0x9a654321
#define ELEM                    1024
/*  #define ELEM                    32768 */


unsigned int next_random(
  unsigned int *number)
{
  unsigned int newbit;

  newbit = ((0x80000000 & *number) >> 31) ^ ((0x10000000 & *number) >> 28);
  *number = (*number << 1) | newbit;

  return(*number);
} /* next_random */



void init_table(
  unsigned int *table,
  unsigned int start,
  unsigned int end)
{
  unsigned int n = SEED, i;

  for(i = start; i <= end; i++)
    table[i] = 0x3ff & next_random(&n);
} /* init_table */



void qsort(
  unsigned int *table,
  int start, 
  int end)
{
  int i, j;
  unsigned int t, pivot;

  i = start - 1;
  j = end + 1;

  pivot = table[start];

  if (start < end) {
    
    while (i < j) {
      do i++; while (table[i] < pivot);
      do j--; while (table[j] > pivot);
      if (i < j) {
        t = table[i];
        table[i] = table[j];
        table[j] = t;
      } /* if */
    } /* while */
    
    qsort(table, start, j);
    qsort(table, j + 1, end);
  } /* if */

} /* qsort */





int child_func(unsigned int me, int * param)
{
  SS_WORD_TYPE msg;
  unsigned int i = 0;

  while (i < 15) {
    msg = getMessage(10000);
    fprintf(stderr, "[%d] (%d) catched an interrupt [%d] !\n",
            me, i++, msg);
    fflush(stderr);
  } /* while */  

  return(msg);
} /* child_func */





void main(int argc, char **argv)
{
  unsigned int me = 0;          /* context id for myself */
  unsigned int i,j;
  SS_WORD_TYPE msg;             /* message to transmit/receive */
  SS_WORD_TYPE sp[8], pc[8];
  int *stack;
  unsigned int *table;
  int a;


  fprintf(stderr, "[%d] %s begin\n", me, EXEC_NAME);
  fprintf(stderr, "[%d] initializing stack\n", me);

  stack = (int *) malloc(8*1024*32*sizeof(int));
  table = (unsigned int *) malloc(ELEM*sizeof(unsigned int));

  loadMMUconfig(0, 1);

  fprintf(stderr, "[%d] executing context 1\n", me);
  createThread(1, 1, &stack[32*1024], &child_func, &a);

  init_table(table, 1, ELEM);
  table[0] = 0;
  table[ELEM + 1] = 0x3ff;

  fprintf(stderr, "[%d] qsort starting.\n", me);

  qsort(table, 1, ELEM);

  fprintf(stderr, "[%d] qsort finished.\n", me);
  fprintf(stderr, "[%d] thread(%d) = %d\n", me, 1, waitThread(1));

  free(stack);
  free(table);

  fprintf(stderr, "[%d] %s end\n", me, EXEC_NAME);

}


