/*
 * test-interrupt-3.c
 *
 * Dominik Madon, LSL/DI/EPFL, 1998, 1999, 2000
 *
 * The goal of this executable is to check out reaction to
 * hard interrupts by a context. There is one context
 * running an application which is stopped by the interrupt.
 * A service interrupt routine increments a counter and
 * restart the interrupted application.
 *
 *
 */

#include <stdio.h>
#include <string.h>
#include "ssmt-3.h"
#include "bench-interrupt-3-handler.h"

#define EXEC_NAME               "bench-interrupt-3"

#define FALSE                   0
#define TRUE                    1

#define SEED                    0x9a654321
#define ELEM                    1024




unsigned int next_random(
  unsigned int *number)
{
  unsigned int newbit;

  newbit = ((0x80000000 & *number) >> 31) ^ ((0x10000000 & *number) >> 28);
  *number = (*number << 1) | newbit;

  return(*number);
} /* next_random */



void init_table(
  unsigned int *table,
  unsigned int start,
  unsigned int end)
{
  unsigned int n = SEED, i;

  for(i = start; i <= end; i++)
    table[i] = 0x3ff & next_random(&n);
} /* init_table */



void qsort(
  unsigned int *table,
  int start, 
  int end)
{
  int i, j;
  unsigned int t, pivot;

  i = start - 1;
  j = end + 1;

  pivot = table[start];

  if (start < end) {
    
    while (i < j) {
      do i++; while (table[i] < pivot);
      do j--; while (table[j] > pivot);
      if (i < j) {
        t = table[i];
        table[i] = table[j];
        table[j] = t;
      } /* if */
    } /* while */
    
    qsort(table, start, j);
    qsort(table, j + 1, end);
  } /* if */

} /* qsort */




void main(int argc, char **argv)
{
  unsigned int me = 0;          /* context id for myself */
  unsigned int i,j;
  SS_WORD_TYPE msg;             /* message to transmit/receive */
  SS_WORD_TYPE sp[8], pc[8];
  int *stack;
  unsigned int *table;
  int a[32];


  fprintf(stderr, "[%d] %s begin\n", me, EXEC_NAME);
  fprintf(stderr, "[%d] a = 0x%08x..0x%08x\n", me, a, &(a[31]));

  for(i = 0; i < 32; i++)
    a[i] = 0;

  fprintf(stderr, "[%d] preparing interrupt vector\n", me);
  asm("lcu %0, %1" :: "r" (&interrupt_handler), "r" (3));

  fprintf(stderr, "[%d] initializing stack\n", me);

  stack = (int *) malloc(8*1024*32*sizeof(int));
  table = (unsigned int *) malloc((ELEM+2)*sizeof(int));

/*    loadMMUconfig(0, 1); */

/*    fprintf(stderr, "[%d] executing context 1\n", me); */
/*    createThread(1, 1, &stack[32*1024], &child_func, &a); */

/*    fprintf(stderr, "[%d] thread(%d) = %d\n", me, 1, waitThread(1)); */

  init_table(table, 1, ELEM);
  table[0] = 0;
  table[ELEM + 1] = 0x3ff;

  fprintf(stderr, "[%d] qsort starting.\n", me);

  qsort(table, 1, ELEM);

  fprintf(stderr, "[%d] qsort finished.\n", me);

  fprintf(stderr, "[%d] disabling interrupt vector\n", me);
  asm("lcu %0, %1" :: "r" (0), "r" (3));

  fprintf(stderr, "[%d] Dumping array a.\n", me);
  for(i = 0; i < 16; i++)
    fprintf(stderr, "[%d] a[%02d] = 0x%08x, a[%02d] = 0x%08x\n",
            me, i, a[i], i+16, a[i+16]);

  fprintf(stderr, "[%d] freeing stack and table.\n", me);

  free(stack);
  free(table);

  fprintf(stderr, "[%d] %s end\n", me, EXEC_NAME);
/*    interrupt_handler(); */

}


