/*
 * bench-lock-11.c
 *
 * Dominik Madon, LSL/DI/EPFL, 1998, 1999
 *
 * The goal of this executable is to time 
 * entering and leaving a critical section
 * using different moethods of mutual exclusion.
 * 
 * The stuffing conctexts are sorting tables.
 *
 */

#include <stdio.h>
#include <string.h>
#include "ssmt-3.h"


#define EXEC_NAME               "bench-lock-11"

#define FALSE                   0
#define TRUE                    1

#define SEED                    0x9a654321
#define ELEM                    64

#define GROUP                   5



unsigned int next_random(
  unsigned int *number)
{
  unsigned int newbit;

  newbit = ((0x80000000 & *number) >> 31) ^ ((0x10000000 & *number) >> 28);
  *number = (*number << 1) | newbit;

  return(*number);
} /* next_random */





void init_table(
  unsigned int *table,
  unsigned int start,
  unsigned int end,
  unsigned int seed)
{
  unsigned int n = seed, i;

  for(i = start; i <= end; i++)
    table[i] = 0x3ff & next_random(&n);
} /* init_table */







void Clock()
{
  loadContextUnitReg(1, 2);
}



void Cunlock()
{
  loadContextUnitReg(0, 2);
}



static unsigned int waiting[32];
static unsigned int lockfree;



void Tlock(unsigned int me)
{
  loadContextUnitReg(1, 2); /* Clock(); */
  if (lockfree) 
    lockfree = FALSE;
  else
    waiting[me] = TRUE;
  if (waiting[me]) {
    loadContextUnitReg(0, 2); /* Cunlock(); */
    loadContextUnitReg(0, 0); /* stop myself */
  } else {
    loadContextUnitReg(0, 2); /* Cunlock(); */
  }
} /* Tlock */



void Tunlock(unsigned int me)
{
  int i;
  SS_WORD_TYPE pc;

  loadContextUnitReg(1, 2); /* Clock(); */
  lockfree = TRUE;
  for(i = 0; i < 32; i++) {
    if (waiting[i]) {
      waiting[i] = FALSE;
      lockfree = FALSE;
      loadContextUnitReg(0, 2); /* Cunlock(); */
      while(activeContext_p(i));
      pc = getPCandStop(i);
      loadPCandResume(pc, i);
      break;
    } /* if */
  } /* for */

  if (lockfree)
    loadContextUnitReg(0, 2); /* Cunlock(); */
  
} /* Tunlock */


int lock_child(unsigned int me, int * param)
{
  unsigned int limit = (unsigned int) param[0];
  unsigned int *next_prime = (unsigned int *) param[1];
  unsigned int *max_index = (unsigned int *) param[2];
  unsigned int *prime = (unsigned int *) param[3];
  unsigned int i, j, k, l, t;
  unsigned int prime_p[5];
  unsigned int p;
  unsigned int tmp;
  unsigned int lnum;

  lnum = me + 10000;

  do {
/*      loadContextUnitReg(1, 2);  */
    lock(lnum, 4); 
/*      Tlock(me); */
      i = *next_prime;
      *next_prime += 5;
      k = *max_index;
/*      loadContextUnitReg(0, 2);  */
    unlock(4);
/*      Tunlock(me); */
    for(l = 0; l < GROUP; l++) {
      prime_p[l] = TRUE;
      j = 0;
      while (prime_p[l] && (j < k)) {
        p = prime[j];
        if (((i + l) % p) == 0)
          prime_p[l] = FALSE;
        j++;
      }
      for(t = 0; t < l; t++) {
        if (((i + l) % (i + t)) == 0)
          prime_p[l] = FALSE;
      }
    }
    t = 0;
    for(l = 0; l < GROUP; l++)
      if (prime_p[l]) t++;
    if (t) {
      lock(lnum, 4);
/*        loadContextUnitReg(1, 2);  */
/*        Tlock(me); */
      for(l = 0; l < GROUP; l++)
        if (prime_p[l])
          prime[++(*max_index)] = i + l;
      unlock(4);
/*        loadContextUnitReg(0, 2); */
/*        Tunlock(me); */
    }
  } while (i < limit); /* (k < limit); */

  return(0);

} /* lock_child */




int sort_child(unsigned int me, int * param)
{
  unsigned int start = (unsigned int) param[0];
  unsigned int stop = (unsigned int) param[1];
  unsigned int seed = (unsigned int) param[2];
  unsigned i, j, tmp;
  unsigned int table[ELEM];

  init_table(table, start, stop, seed);
  for(i = start; i < stop; i++) {
    for(j = i; j < stop; j++) {
      if (table[i] > table[j]) {
        tmp = table[i];
        table[i] = table[j];
        table[j] = tmp;
      }
    }
  }

  return 0;

} /* sort_child */



void main(int argc, char **argv)
{
  unsigned int me = 0;          /* context id for myself */
  unsigned int i,j;
  int *stack;

  unsigned int param;

  int param_child_A[4];
  int param_child_B[4];
  int param_child_C[4];
  int param_child_D[4];
  int param_child_E[4];
  int param_child_F[4];

  unsigned int prime[4096];
  unsigned int max_index;
  unsigned int next_prime;

  fprintf(stderr, "[%d] %s begin\n", me, EXEC_NAME);

  if (argc > 1) {
    fprintf(stderr, "[%d] reading parameter... ", me);    
    i = 0;
    param = 0;
    while (argv[1][i]) {
      param = (param * 10) + (argv[1][i++] - '0');
    }
    fprintf(stderr, "%d\n", param);
  } else {
    fprintf(stderr, "[%d] Missing positive parameter\n", me);    
  }

  fprintf(stderr, "[%d] initializing stack\n", me);
  fflush(stderr);
  stack = (int *) malloc(1024*256*sizeof(int));

  for(i = 0; i < 32 ; i++)
    waiting[i] = FALSE;
  lockfree = TRUE;

  prime[0] = 2;
  prime[1] = 3;
  max_index = 1;
  next_prime = 4;

  loadMMUconfig(0, 1);
  loadMMUconfig(0, 2);
  loadMMUconfig(0, 3);
  loadMMUconfig(0, 4);
  loadMMUconfig(0, 5);
  loadMMUconfig(0, 6);

  param_child_A[0] = param;
  param_child_B[0] = param;
  param_child_C[1] = 0;
  param_child_D[1] = 0;
  param_child_E[1] = 0;
  param_child_F[1] = 0;
  param_child_A[1] = (unsigned int) &next_prime;
  param_child_B[1] = (unsigned int) &next_prime;
  param_child_C[1] = ELEM - 1;
  param_child_D[1] = ELEM - 1; 
  param_child_E[1] = ELEM - 1;
  param_child_F[1] = ELEM - 1; 
  param_child_A[2] = (unsigned int) &max_index;
  param_child_B[2] = (unsigned int) &max_index;
  param_child_C[2] = SEED >> 3;
  param_child_D[2] = SEED >> 7;
  param_child_E[2] = SEED >> 11;
  param_child_F[2] = SEED >> 17;
  param_child_A[3] = (unsigned int) prime;
  param_child_B[3] = (unsigned int) prime;

  createThread(3, 3, &(stack[96*1024]), &sort_child, param_child_C);
/*    createThread(4, 4, &(stack[128*1024]), &sort_child, param_child_D); */
/*    createThread(5, 5, &(stack[160*1024]), &sort_child, param_child_E); */
/*    createThread(6, 6, &(stack[192*1024]), &sort_child, param_child_F); */

  createThread(1, 1, &(stack[32*1024]), &lock_child, param_child_A);
  createThread(2, 2, &(stack[64*1024]), &lock_child, param_child_B);

  waitThread(3);
/*    waitThread(4); */
/*    waitThread(5); */
/*    waitThread(6); */

  Clock();
    next_prime = param + 1;
  Cunlock();

  waitThread(1);
  waitThread(2);

/*    for (i = 0; i <= max_index; i += 4) */
/*      fprintf(stderr, "[%d] %4d %4d %4d %4d\n", me, prime[i],  */
/*              prime[i + 1], prime[i + 2], prime[i + 3]); */
  fprintf(stderr, "[%d] max_prime = %d\n", me, prime[max_index]);
  free(stack);

  fprintf(stderr, "[%d] %s end\n", me, EXEC_NAME);

}


