/*
 * bench-lock-3.c
 *
 * Dominik Madon, LSL/DI/EPFL, 1998, 1999
 *
 * The goal of this executable is to time 
 * locks implemented with active loops.
 * 
 * The executable is retreiving prime number
 * using Erethostenus method. Two contexts
 * build an array of prime numbers. When writing
 * in it they need to lock the entry before 
 * writing to them.
 *
 */

#include <stdio.h>
#include <string.h>
#include "ssmt-3.h"


#define EXEC_NAME               "bench-lock-3"

#define FALSE                   0
#define TRUE                    1

#define SEED                    0x9a654321




static unsigned int ok[2];
static unsigned int turn;





void Tlock(unsigned int id)
{
  ok[1 - id] = TRUE;
  turn = 1 - id;
  while (ok[id] && (turn != id));
}



void Tunlock(unsigned int id)
{
  ok[1 - id] = FALSE;
}






int child(unsigned int me, int * param)
{
  unsigned int limit = (unsigned int) param[0];
  unsigned int *next_prime = (unsigned int *) param[1];
  unsigned int *max_index = (unsigned int *) param[2];
  unsigned int *prime = (unsigned int *) param[3];
  unsigned int i, j, k;
  unsigned int prime_p;
  unsigned int p;
  unsigned int tmp;


  do {
    Tlock(me - 1);
      i = (*next_prime)++;
      k = *max_index;
    Tunlock(me - 1);
    prime_p = TRUE;
    j = 0;
    while (prime_p && (j < k)) {
      p = prime[j];
      if ((i % p) == 0)
        prime_p = FALSE;
      j++;
    }
    if (prime_p) {
      Tlock(me - 1);
        while (++k <= *max_index) {
          if (prime[k] > i) {
            tmp = prime[k];
            prime[k] = i;
           i = tmp;
          }
        }
        prime[k] = i;
        *max_index = k;
      Tunlock(me - 1);      
    }
  } while (i < limit); /* (k < limit); */

  return(0);

} /* child */







void main(int argc, char **argv)
{
  unsigned int me = 0;          /* context id for myself */
  unsigned int i,j;
  int *stack;

  unsigned int param;

  int param_child_A[4];
  int param_child_B[4];
/*    int param_child_C[4]; */
/*    int param_child_D[4]; */

  unsigned int prime[1024];
  unsigned int max_index;
  unsigned int next_prime;

  fprintf(stderr, "[%d] %s begin\n", me, EXEC_NAME);

  if (argc > 1) {
    fprintf(stderr, "[%d] reading parameter... ", me);    
    i = 0;
    param = 0;
    while (argv[1][i]) {
      param = (param * 10) + (argv[1][i++] - '0');
    }
    fprintf(stderr, "%d\n", param);
  } else {
    fprintf(stderr, "[%d] Missing positive parameter\n", me);    
  }

  fprintf(stderr, "[%d] initializing stack\n", me);
  fflush(stderr);
  stack = (int *) malloc(1024*64*sizeof(int));


  prime[0] = 2;
  prime[1] = 3;
  max_index = 1;
  next_prime = 4;

  loadMMUconfig(0, 1);
  loadMMUconfig(0, 2);
/*    loadMMUconfig(0, 3); */
/*    loadMMUconfig(0, 4); */

  param_child_A[0] = param;
  param_child_B[0] = param;
/*    param_child_C[0] = SEED >> 4; */
/*    param_child_D[0] = SEED << 2; */
  param_child_A[1] = (unsigned int) &next_prime;
  param_child_B[1] = (unsigned int) &next_prime;
    /*  param_child_C[1] = */
/*      param_child_D[1] = param; */ 
  param_child_A[2] = (unsigned int) &max_index;
  param_child_B[2] = (unsigned int) &max_index;
/*    param_child_C[2] = 2; */
/*    param_child_D[2] = 3; */
  param_child_A[3] = (unsigned int) prime;
  param_child_B[3] = (unsigned int) prime;

  ok[0] = FALSE;
  ok[1] = FALSE;
  turn = 0;

  createThread(1, 1, &(stack[16*1024]), &child, param_child_A);
  createThread(2, 2, &(stack[32*1024]), &child, param_child_B);
/*    createThread(3, 3, &(stack[192*1024]), &child, param_child_C); */
/*    createThread(4, 4, &(stack[256*1024]), &child, param_child_D); */

/*    for(i = 0; i < 32 ; i++) */
/*      asm("nop"); */

  waitThread(1);
  waitThread(2);

/*    for (i = 0; i <= max_index; i += 4) */
/*      fprintf(stderr, "[%d] %4d %4d %4d %4d\n", me, prime[i],  */
/*              prime[i + 1], prime[i + 2], prime[i + 3]); */
/*    fprintf(stderr, "[%d] next_prime = %d\n", me, next_prime);  */
/*    waitThread(3); */
/*    waitThread(4); */

  free(stack);

  fprintf(stderr, "[%d] %s end\n", me, EXEC_NAME);

}


