/*
 * bench-lock-6.c
 *
 * Dominik Madon, LSL/DI/EPFL, 1998, 1999
 *
 * The goal of this executable is to time 
 * mutual exclusion by context exclusion.
 * 
 * The executable is retreiving prime number
 * using Erethostenus method. Two contexts
 * build an array of prime numbers. When writing
 * in it they need to lock the entry before 
 * writing to them.
 *
 */

#include <stdio.h>
#include <string.h>
#include "ssmt-3.h"


#define EXEC_NAME               "bench-lock-6"

#define FALSE                   0
#define TRUE                    1

#define SEED                    0x9a654321




void Clock()
{
  loadContextUnitReg(1, 2);
}



void Cunlock()
{
  loadContextUnitReg(0, 2);
}



int child(unsigned int me, int * param)
{
  unsigned int limit = (unsigned int) param[0];
  unsigned int *next_prime = (unsigned int *) param[1];
  unsigned int *max_index = (unsigned int *) param[2];
  unsigned int *prime = (unsigned int *) param[3];
  unsigned int i, j, k;
  unsigned int prime_p;
  unsigned int p;
  unsigned int tmp;

  do {
    Clock();
      i = (*next_prime)++;
      k = *max_index;
    Cunlock();
    prime_p = TRUE;
    j = 0;
    while (prime_p && (j < k)) {
      p = prime[j];
      if ((i % p) == 0)
        prime_p = FALSE;
      j++;
    }
    if (prime_p) {
      Clock();
        while (++k <= *max_index) {
          if (prime[k] > i) {
            tmp = prime[k];
            prime[k] = i;
            i = tmp;
          }
        }
        prime[k] = i;
        *max_index = k;
      Cunlock();      
    }
  } while (i < limit); /* (k < limit); */

  return(0);

} /* child */







void main(int argc, char **argv)
{
  unsigned int me = 0;          /* context id for myself */
  unsigned int i,j;
  int *stack;

  unsigned int param;

  int param_child_A[4];
  int param_child_B[4];
  int param_child_C[4];
  int param_child_D[4];
  int param_child_E[4];
  int param_child_F[4];

  unsigned int prime[1024];
  unsigned int max_index;
  unsigned int next_prime;

  fprintf(stderr, "[%d] %s begin\n", me, EXEC_NAME);

  if (argc > 1) {
    fprintf(stderr, "[%d] reading parameter... ", me);    
    i = 0;
    param = 0;
    while (argv[1][i]) {
      param = (param * 10) + (argv[1][i++] - '0');
    }
    fprintf(stderr, "%d\n", param);
  } else {
    fprintf(stderr, "[%d] Missing positive parameter\n", me);    
  }

  fprintf(stderr, "[%d] initializing stack\n", me);
  fflush(stderr);
  stack = (int *) malloc(1024*256*sizeof(int));


  prime[0] = 2;
  prime[1] = 3;
  max_index = 1;
  next_prime = 4;

  loadMMUconfig(0, 1);
  loadMMUconfig(0, 2);
  loadMMUconfig(0, 3);
  loadMMUconfig(0, 4);
  loadMMUconfig(0, 5);
  loadMMUconfig(0, 6);

  param_child_A[0] = param;
  param_child_B[0] = param;
  param_child_C[0] = param;
  param_child_D[0] = param;
  param_child_E[0] = param;
  param_child_F[0] = param;
  param_child_A[1] = (unsigned int) &next_prime;
  param_child_B[1] = (unsigned int) &next_prime;
  param_child_C[1] = (unsigned int) &next_prime;
  param_child_D[1] = (unsigned int) &next_prime; 
  param_child_E[1] = (unsigned int) &next_prime;
  param_child_F[1] = (unsigned int) &next_prime; 
  param_child_A[2] = (unsigned int) &max_index;
  param_child_B[2] = (unsigned int) &max_index;
  param_child_C[2] = (unsigned int) &max_index;
  param_child_D[2] = (unsigned int) &max_index;
  param_child_E[2] = (unsigned int) &max_index;
  param_child_F[2] = (unsigned int) &max_index;
  param_child_A[3] = (unsigned int) prime;
  param_child_B[3] = (unsigned int) prime;
  param_child_C[3] = (unsigned int) prime;
  param_child_D[3] = (unsigned int) prime;
  param_child_E[3] = (unsigned int) prime;
  param_child_F[3] = (unsigned int) prime;

  createThread(1, 1, &(stack[16*1024]), &child, param_child_A);
  createThread(2, 2, &(stack[32*1024]), &child, param_child_B);
  createThread(3, 3, &(stack[48*1024]), &child, param_child_C);
  createThread(4, 4, &(stack[64*1024]), &child, param_child_D);
  createThread(5, 5, &(stack[80*1024]), &child, param_child_E);
  createThread(6, 6, &(stack[96*1024]), &child, param_child_F);

  waitThread(1);
  waitThread(2);
  waitThread(3);
  waitThread(4);
  waitThread(5);
  waitThread(6);

/*    for (i = 0; i <= max_index; i += 4) */
/*      fprintf(stderr, "[%d] %4d %4d %4d %4d\n", me, prime[i],  */
/*              prime[i + 1], prime[i + 2], prime[i + 3]); */

  free(stack);

  fprintf(stderr, "[%d] %s end\n", me, EXEC_NAME);

}


