/*
 * bench-philo-1.c
 *
 * Dominik Madon, LSL/DI/EPFL, 1998, 1999, 2000.
 *
 * Ce test utilise les verrous pour rserver les fourchettes
 * des philosophes.
 *
 */

#include <stdio.h>
#include "ssmt-3.h"


#define EXEC_NAME       "bench-philo-1"
#define NUM_CXT         8

#define FALSE           0
#define TRUE            1

/* Germe de depart pour le generateur de nombre alatoire */
#define SEED            0x9a654321

/* Nombre de bouche par dfaut */
#define BITES           3

/* Information transmise  tous les contextes */
typedef struct {
  unsigned int random;
  unsigned int counter;
} PARAM_T;



/*
 * Gnrateur de nombre alatoire bas sur un dcalage de bits
 * vers la gauche et insertion du XOR entre les bits 0 et 3
 * de la chane initiale.
 */

unsigned int next_random(
  unsigned int *number)
{
  unsigned int newbit;

  newbit = ((0x80000000 & *number) >> 31) ^ ((0x10000000 & *number) >> 28);
  *number = (*number << 1) | newbit;

  return(*number);
} /* next_random */



/*
 * Procdure de gestion des philosophes.
 *
 * me vaut le numro de thread (quivalent
 * ici au numro de contexte).
 */

int philosophe(unsigned int me, PARAM_T * param)
{
  unsigned int right, left, next_right_port, next_left_port;
  unsigned int random = param->random + me;
  unsigned int counter = param->counter;
  unsigned int i, j, k = 9;

  /* Verrous utiliser pour obtenir les fourchettes */

  right = ((me + 1) % NUM_CXT);
  left = me;

  /* Numros des ports d'attente */

  next_right_port = 9900 + me;
  next_left_port = 9800 + me;

  /* boucle principale, alternant la prise de fourchettes, */
  /* l'alimentation et la rflexion. */

  for(j = counter; j > 0; j--) {

    /* Prise de fourchette */

    if (me % 2 == 0) {
      lock(next_right_port, right); /* Fourchette de droite */
      lock(next_left_port, left); /* Fourchette de gauche */
    } else {
      lock(next_left_port, left); /* Fourchette de gauche */
      lock(next_right_port, right); /* Fourchette de droite */
    } /* if */

    /* Alimentation... */
    
    unlock(left); 
    unlock(right);

    /* Attente (le philosophe rflchit)... */

    fprintf(stderr,".");

    next_random(&random);
    for(i = (0xf & random) * 100; i > 0; i--) {
      if ((i % 100) == 0) {
        k = (k + (i * 3)) % 0x1000000;
      }
    } /* for */

  } /* for */

  return((SS_WORD_TYPE) 0);

} /* philosophe */




/*
 * Corps principal
 */

void main(int argc, char **argv)
{
  SS_CONTEXT_TYPE me = 0, context;
  int *stack;
  unsigned int max_cxt = 1;
  PARAM_T *a;


  fprintf(stderr, "[%d] %s begin\n", me, EXEC_NAME);

  /* Prparation des arguments pour les philosphes. */
  /* ventuellement, le nombre de bouches est donn */
  /* par la ligne de commande */

  a = (PARAM_T *) malloc(sizeof(PARAM_T));

  if (argc > 1) {
    sscanf(argv[1], "%d", &(a->counter));

    if (a->counter < 1)
      a->counter = BITES;
  } else {
    a->counter = BITES;
  }
  a->random = SEED;

  /* Prparation des piles locales  chaque contexte */

  fprintf(stderr, "[%d] initializing stack\n", me);
  stack = (int *) malloc(NUM_CXT * 2048 * sizeof(int));

  /* Prparation de la configuration MMU en espace */
  /* d'adressage commun  tous les threads. */

  fprintf(stderr, "[%d] initializing MMU config\n", me);
  for(context = 1; context < NUM_CXT; context++) {
    loadMMUconfig(0, context);
  } /* for */

  /* Lancements des threads sur les contextes. */

  for(context = 1; context < NUM_CXT; context++) {
    createThread(context,                       /* threadID */
                 context,                       /* numro de contexte */
                 &stack[(context * 1024) - 1],  /* pile du thread */
                 &philosophe,                   /* fonction  excuter */
                 (void *) a);                   /* paramtres de la fonction */
  } /* for */

  /* et excution par le contexte lanceur d'un philosophe */

  philosophe(me, (void *) a);

  /* Attente de la fin de l'excution de tous les contextes */

  for(context = 1; context < NUM_CXT; context++) {
    waitThread(context);
  } /* for */

  /* Libration des piles locales */

  free(stack);

  fflush(stdout);
  fprintf(stdout, "\n[%d]%s end\n", me, EXEC_NAME);

}


