/*
 * bench-swap-4.c
 *
 * Dominik Madon, LSL/DI/EPFL, 1998, 1999
 *
 * The goal of this executable is to test swapping context
 * in/out. Five processes calculating fibonacci's numbers are run
 * concurrently on 4 contexts. A sixth processes is swapping among the
 * five processes and four contexts.
 *
 * The number of swap in/out can be choosen given a parameter.
 * For exemple, given 10 will make 100 swap in/out and a waiting loop
 * 500 iterations. 
 *
 */

#include <stdio.h>
#include <string.h>
#include "ssmt-3.h"


#define EXEC_NAME               "bench-swap-4"

#define FALSE                   0
#define TRUE                    1



int child_func(unsigned int me, int * param)
{
  SS_WORD_TYPE msg;
  int prime[2000];
  unsigned int i, j, index, max, prime_p;
  
  prime[0] = 1;
  prime[1] = 2;
  prime[2] = 3;
  index = 2;

  for(i = 4; index < 1000; i++) {
    max = i / 2;
    prime_p = TRUE;
    j = 1;
    do {
      if ((i % prime[j]) == 0)
        prime_p = FALSE;
      j++;
    } while ((j < index) && (prime_p));
    do {
      j = prime[index] + 1;
      if ((i % j) == 0)
        prime_p = FALSE;
      j++;
    } while ((j <= max) && (prime_p));
    if (prime_p) {
      prime[++index] = i;
    } /* if */
  } /* for */

  return(prime[index - 1]);
} /* child_func */





unsigned int running(SS_CONTEXT_TYPE context)
{
  unsigned int mask;

  mask = getContextUnitReg(0);

  return(mask & (1 << context));
}




void main(int argc, char **argv)
{
  unsigned int me = 0;          /* context id for myself */
  unsigned int i,j;
  SS_WORD_TYPE msg;             /* message to transmit/receive */
  SS_WORD_TYPE sp[8], pc[8];
  int *stack;
  int a, b, c;
  int d, e, f;
  SS_CONTEXT_TYPE in[5] = {0, 1, 2, 3, 4};
  SS_CONTEXT_TYPE out = 5;
  SS_CONTEXT_TYPE temp;
  unsigned int where = 2;
  unsigned int waiting_iter = 1000;
  unsigned int swaps = 0;

  fprintf(stderr, "[%d] %s begin\n", me, EXEC_NAME);
  fprintf(stderr, "[%d] initializing stack\n", me);

  stack = (int *) malloc(128*1024*32*sizeof(int));

  loadMMUconfig(0, 1);
  loadMMUconfig(0, 2);
  loadMMUconfig(0, 3);
  loadMMUconfig(0, 4);

  a = 10;
  b = 20;
  c = 30;
  d = 10;
  e = 20;
  f = 30;

  if (argc > 1) {
    i = 0;
    waiting_iter = 0;
    while (argv[1][i]) {
      waiting_iter = (waiting_iter * 10) + (argv[1][i++] - '0');
    }
  }

  fprintf(stderr, "[%d] waiting iter=%d\n", me, waiting_iter);

  fprintf(stderr, "[%d] executing context 5\n", me);
  createThread(5, 1, &stack[128*1024], &child_func, &a);
  fprintf(stderr, "[%d] executing context 2\n", me);
  createThread(2, 2, &stack[256*1024], &child_func, &b);
  fprintf(stderr, "[%d] executing context 3\n", me);
  createThread(3, 3, &stack[512*1024], &child_func, &c);
  fprintf(stderr, "[%d] executing context 4\n", me);
  createThread(4, 4, &stack[1024*1024], &child_func, &d);

  fprintf(stderr, "[%d] threadout(5)\n", me);
  swapThreadOut(1, &sp[5], &pc[5]);
  fprintf(stderr, "[%d] thread(5) stopped (pc=0x%08x sp=0x%08x)\n", 
          me, pc[5], sp[5]);

  fprintf(stderr, "[%d] executing context 1\n", me);
  createThread(1, 1, &stack[1156*1024], &child_func, &f);

  while(running(where)) {
    fprintf(stderr, "[%d] swap out thread %d in context %d\n", 
            me, in[where], where);
    if (running(where))
      swapThreadOut(where, &sp[in[where]], &pc[in[where]]);
    else
      break;
    fprintf(stderr, "[%d] swap in thread %d in context %d\n", 
            me, out, where);
    swapThreadIn(where, sp[out], pc[out]);

    temp = out;
    out = in[where];
    in[where] = temp;
    where = (where % 4) + 1;

    swaps++;

    for(j = 0; j < waiting_iter; j++);
  }

   fprintf(stderr, "[%d] thread(%d) = %d\n", me, in[where],
          waitThread(in[where]));
  for(j = 0; j < 5000; j++);
  fprintf(stderr, "[%d] swap in thread %d in context %d\n", 
          me, out, where);
  for(j = 0; j < 5000; j++);
  swapThreadIn(where, sp[out], pc[out]);
  for(i = 0; i < 4; i++) {
    temp = out;
    out = in[where];
    in[where] = temp;
    where = (where % 4) + 1;
    fprintf(stderr, "[%d] thread(%d) = %d\n", me, in[where],
            waitThread(in[where]));
  }    

  free(stack);

  fprintf(stderr, "[%d] Total number of swaps: %d\n", me, swaps);
  fprintf(stderr, "[%d] %s end\n", me, EXEC_NAME);

}


