/*
 * bench-swap-1.c
 *
 * Dominik Madon, LSL/DI/EPFL, 1998, 1999
 *
 * The goal of this executable is to test swapping context
 * in/out. Five processes calculating fibonacci's numbers are run
 * concurrently on 4 contexts. A sixth processes is swapping among the
 * five processes and four contexts.
 *
 * */

#include <stdio.h>
#include <string.h>
#include "ssmt-3.h"


#define EXEC_NAME               "bench-swap-1"

#define FALSE                   0
#define TRUE                    1



int child_func(unsigned int me, int * param)
{
  SS_WORD_TYPE msg;
  int prime[180];
  unsigned int i, j, index, max, prime_p;
  
  prime[0] = 1;
  prime[1] = 2;
  prime[2] = 3;
  index = 2;

  for(i = 4; index < 180; i++) {
    max = i / 2;
    prime_p = TRUE;
    j = 1;
    do {
      if ((i % prime[j]) == 0)
        prime_p = FALSE;
      j++;
    } while ((j < index) && (prime_p));
    do {
      j = prime[index] + 1;
      if ((i % j) == 0)
        prime_p = FALSE;
      j++;
    } while ((j <= max) && (prime_p));
    if (prime_p) {
      prime[++index] = i;
    } /* if */
  } /* for */

  return(prime[index - 1]);
} /* child_func */





void main(int argc, char **argv)
{
  unsigned int me = 0;          /* context id for myself */
  unsigned int i,j;
  SS_WORD_TYPE msg;             /* message to transmit/receive */
  SS_WORD_TYPE sp[8], pc[8];
  int *stack;
  int a[8], *b;
  SS_CONTEXT_TYPE out;
  SS_CONTEXT_TYPE in;
  unsigned int where;
  unsigned thread_num;

  fprintf(stderr, "[%d] %s begin\n", me, EXEC_NAME);

  if (argc > 1) {
    thread_num = (argv[1][0] - '0');
  } else {
    thread_num = 2;
  }

  fprintf(stderr, "[%d] running for %d context\n", me, thread_num);
  fprintf(stderr, "[%d] initializing stack\n", me);

  stack = (int *) malloc(128*1024*32*sizeof(int));

  for(i = 0; i < thread_num; i++) {
    loadMMUconfig(0, i);
    a[i] = 10 * (1 + i);
  } /* for */
  b = a;

  where = 1;
  for(i = 1; i < thread_num; i++) {
    fprintf(stderr, "[%d] executing context %d\n", me, i+1);
    createThread(i, where, &stack[i*64*1024], &child_func, b++);
    where = 1 + where % (thread_num - 1);
  }
  in = thread_num;
  out = 1 + in % thread_num;

  for(j = 0; j < 5000; j++);

  fprintf(stderr, "[%d] swap out thread %d in context %d\n", 
          me, out, where);

  swapThreadOut(where, &sp[out], &pc[out]);
/*    fprintf(stderr, "[%d] thread() stopped (pc=0x%08x sp=0x%08x)\n",  */
/*            me, pc[5], sp[5]); */

  fprintf(stderr, "[%d] swap in thread %d in context %d\n", 
          me, in, where);
  createThread(in, where, &stack[in*64*1024], &child_func, b++);
  
  for(j = 0; j < 5000; j++);
/*    for(i = 0; i < 50; i++) { */
  for(i = 0; i < 10; i++) {
    where = 1 + where % (thread_num - 1);
    in = out;
    out = 1 + in % thread_num;

    fprintf(stderr, "[%d] swap out thread %d in context %d\n", 
            me, out, where);
    swapThreadOut(where, &sp[out], &pc[out]);
    fprintf(stderr, "[%d] swap in thread %d in context %d\n", 
            me, in, where);
    swapThreadIn(where, sp[in], pc[in]);

    for(j = 0; j < 5000; j++);
  }

  for(i = 0; i < thread_num - 1; i++) {
    out = 1 + out % thread_num;
    fprintf(stderr, "[%d] thread(%d) = %d\n", me, 
            out, waitThread(out));
  }
  out = 1 + out % thread_num;
  fprintf(stderr, "[%d] swap in thread %d in context %d\n", 
          me, out, where);
  swapThreadIn(where, sp[out], pc[out]);

  fprintf(stderr, "[%d] thread(%d) = %d\n", me, out, waitThread(out));

  free(stack);

  fprintf(stderr, "[%d] %s end\n", me, EXEC_NAME);

}


