/*
 * test-dump_reg.c
 *
 * Dominik Madon, LSL/DI/EPFL, 1998
 *
 * The goal of this executable is to dump the contents of registers
 * of the second context. The first one reads second's regs and print
 * them.
 *
 */

#include <stdio.h>
#include <string.h>
#include "libssmt.h"


#define EXEC_NAME               "test-dump_regs"

#define FALSE                   0
#define TRUE                    1



void dump_regs(SS_CONTEXT_TYPE context)
{
  int i, line;
  char result[1600], temp[10];

  sprintf(result, "[%d] register dump\n", context);
  for(i = 0; i < SS_REG_MAX; i++) {
    fprintf(stderr, "");
    switch(i) {
    case SS_REG_I0:
      strcat(result, "integer registers\n"); 
      line = 0; 
      break;
    case SS_REG_F0:
      strcat(result, "floating point registers\n");
      line = 0; 
      break;
    case SS_REG_HI:
      strcat(result, "HI/LO registers\n");
      line = 0; 
      break;
    case SS_REG_FCC: 
      strcat(result, "\nFCC register\n");
      line = 0; 
      break;
    case SS_REG_PC: 
      strcat(result, "\nPC register\n");
      line = 0; 
      break;
    case SS_REG_CXTCTRL: 
      strcat(result, "\nCXTCTRL register\n");
      line = 0; 
      break;
    }
    sprintf(temp, "%08x ", readContext(context, i));
    strcat(result, temp);
    if ((++line % 8) == 0)
      strcat(result, "\n");
  } /* for */
  fprintf(stderr, "%s\n", result);
} /* dump_reg */


void dump_ACXT(SS_CONTEXT_TYPE context)
{
  int i;
  char line[100], temp[10];

  sprintf(temp, "[%d]", context);
  strcpy(line, temp);
  strcat(line, " active contexts => ");
  for(i = 31; i >= 0; i--) {
    strcat(line, activeContext_p(i) ? "r" : "s");
  }
  fprintf(stderr, "%s\n", line);

} /* dump_ACXT */




void main(int argc, char **argv)
{
  unsigned int me;              /* context id for myself */
  unsigned int i;
  SS_WORD_TYPE msg;             /* message to transmit/receive */

  me = contextID();

  fprintf(stderr, "[%d] %s begin\n", me, EXEC_NAME);

  if (me == 1) {
    dump_ACXT(me);
    suspendContext(me);
    fprintf(stderr, "[%d] %d path\n", me, 1);
  } else if (me == 2) {
    dump_ACXT(me);
    while (activeContext_p(1));    
    suspendContext(me);
    fprintf(stderr, "[%d] %d path\n", me, 2);
  } else if (me == 0) {
    while (activeContext_p(1));
    while (activeContext_p(2));
    for(i = 0; i < SS_REG_MAX; i++) {
      writeContext(2, i, readContext(1, i));
    }
    dump_ACXT(me);
    fprintf(stderr, "..1..\n");
    resumeContext(1); 
    fprintf(stderr, "..2..\n");
    resumeContext(2); 
/*      dump_ACXT(me); */
    fprintf(stderr, "..3..\n");
    while (!activeContext_p(1));
    fprintf(stderr, "..4..\n");
    while (!activeContext_p(2));
    fprintf(stderr, "..5..\n");
  } /* if */

    dump_ACXT(me); 

  fprintf(stderr, "[%d] %s end\n", me, EXEC_NAME);

}
