/*
 * test-philo.c
 *
 * Dominik Madon, LSL/DI/EPFL, 1998
 *
 *
 *
 */

#include <stdio.h>
#include "libssmt.h"


#define EXEC_NAME               "test-philo"
#define NUM_CXT                 8

#define I_WANT_TO_EAT           1
#define THIS_IS_THE_FORK        2

#define LEFT_FORK               0
#define RIGHT_FORK              1

#define FALSE                   0
#define TRUE                    1

#define BITES                   1000



void serveFork(
  unsigned int fork[2],
  unsigned int me,
  unsigned int right,
  unsigned int left)
{
  SS_WORD_TYPE msg;

  /* Does my right neighbours need my right fork if I
     got it */

  if (fork[RIGHT_FORK] && testMsgFromContext(right)) {
    receiveFromContext(&msg, right);
    if (msg == I_WANT_TO_EAT) {
      sendToContext(THIS_IS_THE_FORK, right);
      fork[RIGHT_FORK] = FALSE;
      fprintf(stderr, "[%d] right fork given to [%d]\n", me, right);
    } else {
      fork[RIGHT_FORK] = TRUE;
      fprintf(stderr, "[%d] got right fork\n", me);
    }
  }

  /* Does my left neighbours need my left fork if I
     got it */
  
  if (fork[LEFT_FORK] && testMsgFromContext(left)) {
    receiveFromContext(&msg, left);
    if (msg == I_WANT_TO_EAT) {
      sendToContext(THIS_IS_THE_FORK, left);
      fork[LEFT_FORK] = FALSE;
      fprintf(stderr, "[%d] left fork given to [%d]\n", me, left);
    } else {
      fork[LEFT_FORK] = TRUE;
      fprintf(stderr, "[%d] got left fork.\n", me);
    }
  }

  /* Does my left neighbours need my left fork if I
     got it */
  
  if (fork[LEFT_FORK] && testMsgFromContext(left)) {
    receiveFromContext(&msg, left);
    if (msg == I_WANT_TO_EAT) {
      sendToContext(THIS_IS_THE_FORK, left);
      fork[LEFT_FORK] = FALSE;
      fprintf(stderr, "[%d] left fork given to [%d]\n", me, left);
    } else {
      fork[LEFT_FORK] = TRUE;
      fprintf(stderr, "[%d] got left fork.\n", me);
    }    
  }

  /* Does my right neighbours need my right fork if I
     got it */

  if (fork[RIGHT_FORK] && testMsgFromContext(right)) {
    receiveFromContext(&msg, right);
    if (msg == I_WANT_TO_EAT) {
      sendToContext(THIS_IS_THE_FORK, right);
      fork[RIGHT_FORK] = FALSE;
      fprintf(stderr, "[%d] right fork given to [%d]\n", me, right);
    } else {
      fork[RIGHT_FORK] = TRUE;
      fprintf(stderr, "[%d] got right fork\n", me);
    }
  }

} /* serveFork */



void drainFork(
  unsigned int fork[2],
  unsigned int me,
  unsigned int right,
  unsigned int left)
{
  SS_WORD_TYPE msg;

  if (!fork[LEFT_FORK]) {
    fprintf(stderr, "[%d] needs left fork...\n", me);
    sendToContext(I_WANT_TO_EAT, left);
    receiveFromContext(&msg, left);
    if (msg == THIS_IS_THE_FORK) {
      fork[LEFT_FORK] = TRUE;
      fprintf(stderr, "[%d] got left fork\n", me);
    }      
  }
  if (!fork[RIGHT_FORK]) {
    fprintf(stderr, "[%d] needs right fork...\n", me);
    sendToContext(I_WANT_TO_EAT, right);
    receiveFromContext(&msg, right);
    if (msg == THIS_IS_THE_FORK) {
      fork[RIGHT_FORK] = TRUE;
      fprintf(stderr, "[%d] got left fork\n", me);
    }
  }
  
  /* Here we're sure to have both forks */

} /* drainFork */




void main(int argc, char **argv)
{
  unsigned int me, left, right;
  unsigned int fork[2];
  unsigned long int random, counter = BITES;
  unsigned int max_cxt = 1;

  SS_WORD_TYPE msg;
  
  me = contextID();

  fork[RIGHT_FORK] = TRUE;
  fork[LEFT_FORK] = FALSE;


  fprintf(stderr, "[%d] %s begin\n", me, EXEC_NAME);

  if (argc > 1) {
    sscanf(argv[1], "%i", &max_cxt);
    if ((max_cxt > 32) || (max_cxt < 1))
      max_cxt = 1;
    if (argc > 2) {
    sscanf(argv[2], "%i", &counter);
    if (counter < 1)
      counter = BITES;
    }
  }

  /* calculating context numbers of both neighbours */

  right = (me + 1) % max_cxt;
  left = (me + max_cxt - 1) % max_cxt;
  fprintf(stderr, "[%d] left neighbour = [%d], right neighbour = [%d]\n",
          me, left, right);  
  
  /* calculating random thinking time */

  random = (1000 % ((me + 1) * 97 + (me * 3 * me))) * 10;
  fprintf(stderr, "[%d] random = %d\n", me, random);

  /* main loop. thinking and serving neighbours */
  
  for(; counter > 0 ; counter--) {
    while (random--)
      serveFork(fork, me, right, left);

    fprintf(stderr, "[%d] want to eat and got %s fork\n", me,
            fork[LEFT_FORK] && fork[RIGHT_FORK] ? "left and right" :
            (fork[LEFT_FORK] ? "left" : (fork[RIGHT_FORK] ? "right" : "any")));

    while (!fork[LEFT_FORK] || !fork[RIGHT_FORK]) {
      serveFork(fork, me, right, left);
      drainFork(fork, me, right, left);
    } /* while */

    fprintf(stderr, "[%d] eating...\n", me);
    random = (1000 % ((me + 1) * 97 + (me * 3 * me))) * 10;
  } /* for */

  while (fork[LEFT_FORK] || fork[RIGHT_FORK]) {
    serveFork(fork, me, right, left);
  } /* while */

  fprintf(stderr, "[%d]%s end\n", me, EXEC_NAME);

}
