/*
 * test-swap-2.c
 *
 * Dominik Madon, LSL/DI/EPFL, 1998
 *
 * The goal of this executable is to test swapping context in/out.
 * A process is replaced by another process. The, the new one
 * is swapped out and the first one is restored
 *
 */

#include <stdio.h>
#include <string.h>
#include "ssmt-2.h"


#define EXEC_NAME               "test-swap-2"

#define FALSE                   0
#define TRUE                    1



int child_func(unsigned int me, int * param)
{
  SS_WORD_TYPE msg;
  int fib[200];
  unsigned int i;
  
  if ((me == 1) || (me == 2))
    readPort(&msg, (0x3000 + me));

  fib[0] = 1;
  fib[1] = 3;

  for(i = 2; i < 200; i++) {
    fib[i] = fib[i-1] + i - fib[i-2];
  } /* for */

  return(fib[199]);
} /* child_func */



void dump_stack(SS_WORD_TYPE *sp)
{
  int i;

  for(i = -1 ; i >= -66 ; i -= 4)
    fprintf(stderr, "$sp[%03d] 0x%08x 0x%08x 0x%08x 0x%08x \n",
            i*4, sp[i], sp[i-1], sp[i-2], sp[i-3]);
}




void main(int argc, char **argv)
{
  unsigned int me = 0;          /* context id for myself */
  unsigned int i;
  SS_WORD_TYPE msg;             /* message to transmit/receive */
  SS_WORD_TYPE sp[8], pc[8];
  int *stack;
  int a, b, c;
  int d, e, f;


  fprintf(stderr, "[%d] %s begin\n", me, EXEC_NAME);
  fprintf(stderr, "[%d] initializing stack\n", me);

  stack = (int *) malloc(8*1024*32*sizeof(int));

  setMMUconfig(1, 0);
  setMMUconfig(2, 0);
  setMMUconfig(3, 0);
  setMMUconfig(4, 0);
  setMMUconfig(5, 0);
  setMMUconfig(6, 0);

  a = 10;
  b = 20;
  c = 30;
  d = 10;
  e = 20;
  f = 30;

  fprintf(stderr, "[%d] executing context 1\n", me);
  createThread(1, 1, &stack[32*1024], &child_func, &a);
  fprintf(stderr, "[%d] executing context 2\n", me);
  createThread(2, 2, &stack[64*1024], &child_func, &b);
  fprintf(stderr, "[%d] executing context 3\n", me);
  createThread(3, 3, &stack[96*1024], &child_func, &c);
  fprintf(stderr, "[%d] executing context 4\n", me);
  createThread(4, 4, &stack[128*1024], &child_func, &d);
  fprintf(stderr, "[%d] executing context 5\n", me);
  createThread(5, 5, &stack[160*1024], &child_func, &e);

  writePort(2000, 0x3000+1);

  fprintf(stderr, "[%d] threadout(1)\n", me);
  swapThreadOut(1, &sp[1], &pc[1]);
  fprintf(stderr, "[%d] thread(1) stopped (pc=0x%08x sp=0x%08x)\n", 
          me, pc[1], sp[1]);

/*   dump_stack((unsigned int *)sp[1]); */

  fprintf(stderr, "[%d] executing context 6\n", me);
  createThread(6, 1, &stack[192*1024], &child_func, &f);

  writePort(2000, 0x3000+2);

  fprintf(stderr, "[%d] threadout(6)\n", me);
  swapThreadOut(1, &sp[6], &pc[6]);
  fprintf(stderr, "[%d] thread(6) stopped (pc=0x%08x sp=0x%08x)\n", 
          me, pc[6], sp[6]);
/*   dump_stack((unsigned int *)sp[6]); */


  fprintf(stderr, "[%d] threadin(1)\n", me);
  swapThreadIn(1, sp[1], pc[1]);


  fprintf(stderr, "[%d] thread(%d) = %d\n", me, 1, waitThread(1));
  fprintf(stderr, "[%d] thread(%d) = %d\n", me, 2, waitThread(2));
  fprintf(stderr, "[%d] thread(%d) = %d\n", me, 3, waitThread(3));
  fprintf(stderr, "[%d] thread(%d) = %d\n", me, 4, waitThread(4));
  fprintf(stderr, "[%d] thread(%d) = %d\n", me, 5, waitThread(5));

  fprintf(stderr, "[%d] threadin(6)\n", me);
  swapThreadIn(1, sp[6], pc[6]);

  fprintf(stderr, "[%d] thread(%d) = %d\n", me, 6, waitThread(6));

  free(stack);

  fprintf(stderr, "[%d] %s end\n", me, EXEC_NAME);

}
