/*
 * test-swap-4.c
 *
 * Dominik Madon, LSL/DI/EPFL, 1998
 *
 * The goal of this executable is to test swapping context
 * in/out. Five processes calculating fibonacci's numbers are run
 * concurrently on 4 contexts. A sixth processes is swapping among the
 * five processes and four contexts.
 *
 * */

#include <stdio.h>
#include <string.h>
#include "ssmt-3.h"


#define EXEC_NAME               "test-swap-4"

#define FALSE                   0
#define TRUE                    1



int child_func(unsigned int me, int * param)
{
  SS_WORD_TYPE msg;
  int prime[200];
  unsigned int i, j, index, max, prime_p;
  
  prime[0] = 1;
  prime[1] = 2;
  prime[2] = 3;
  index = 2;

  for(i = 4; index < 200; i++) {
    max = i / 2;
    prime_p = TRUE;
    j = 1;
    do {
      if ((i % prime[j]) == 0)
        prime_p = FALSE;
      j++;
    } while ((j < index) && (prime_p));
    do {
      j = prime[index] + 1;
      if ((i % j) == 0)
        prime_p = FALSE;
      j++;
    } while ((j <= max) && (prime_p));
    if (prime_p) {
      prime[++index] = i;
    } /* if */
  } /* for */

  return(prime[index - 1]);
} /* child_func */





void main(int argc, char **argv)
{
  unsigned int me = 0;          /* context id for myself */
  unsigned int i,j;
  SS_WORD_TYPE msg;             /* message to transmit/receive */
  SS_WORD_TYPE sp[8], pc[8];
  int *stack;
  int a, b, c;
  int d, e, f;
  SS_CONTEXT_TYPE in[5] = {0, 1, 2, 3, 4};
  SS_CONTEXT_TYPE out = 5;
  unsigned int where = 2;


  fprintf(stderr, "[%d] %s begin\n", me, EXEC_NAME);
  fprintf(stderr, "[%d] initializing stack\n", me);

  stack = (int *) malloc(8*1024*32*sizeof(int));

  loadMMUconfig(0, 1);
  loadMMUconfig(0, 2);
  loadMMUconfig(0, 3);
  loadMMUconfig(0, 4);
  loadMMUconfig(0, 5);

  a = 10;
  b = 20;
  c = 30;
  d = 10;
  e = 20;
  f = 30;

  fprintf(stderr, "[%d] executing context 5\n", me);
  createThread(5, 1, &stack[32*1024], &child_func, &a);
  fprintf(stderr, "[%d] executing context 2\n", me);
  createThread(2, 2, &stack[64*1024], &child_func, &b);
  fprintf(stderr, "[%d] executing context 3\n", me);
  createThread(3, 3, &stack[96*1024], &child_func, &c);
  fprintf(stderr, "[%d] executing context 4\n", me);
  createThread(4, 4, &stack[128*1024], &child_func, &d);

  fprintf(stderr, "[%d] threadout(5)\n", me);
  swapThreadOut(1, &sp[5], &pc[5]);
  fprintf(stderr, "[%d] thread(5) stopped (pc=0x%08x sp=0x%08x)\n", 
          me, pc[5], sp[5]);

  fprintf(stderr, "[%d] executing context 1\n", me);
  createThread(1, 1, &stack[192*1024], &child_func, &f);

/*    for(i = 0; i < 50; i++) { */
  for(i = 0; i < 200; i++) {
    SS_CONTEXT_TYPE temp;

/*      fprintf(stderr, "[%d] swap out thread %d in context %d\n",  */
/*              me, in[where], where); */
    swapThreadOut(where, &sp[in[where]], &pc[in[where]]);
/*      fprintf(stderr, "[%d] swap in thread %d in context %d\n",  */
/*              me, out, where); */
    swapThreadIn(where, sp[out], pc[out]);

    temp = out;
    out = in[where];
    in[where] = temp;
    where = (where % 4) + 1;

    for(j = 0; j < 250; j++);
  }


  fprintf(stderr, "[%d] thread(%d) = %d\n", me, 1, waitThread(1));
  fprintf(stderr, "[%d] thread(%d) = %d\n", me, 2, waitThread(2));
  fprintf(stderr, "[%d] thread(%d) = %d\n", me, 3, waitThread(3));
  fprintf(stderr, "[%d] thread(%d) = %d\n", me, 4, waitThread(4));

  fprintf(stderr, "[%d] threadin(5)\n", me);
  swapThreadIn(1, sp[5], pc[5]);

  fprintf(stderr, "[%d] thread(%d) = %d\n", me, 5, waitThread(5));

  free(stack);

  fprintf(stderr, "[%d] %s end\n", me, EXEC_NAME);

}


