# ssmt-2.s
#
# Implementation part of the routines to exploit the
# port related functions.       
# 
# Dominik Madon, LSL-EPFL, 1998.
# This file is part of my thesis work.

        .align  2
        .globl  setMMUconfig
        .align  2
        .globl  getMMUconfig
                
       	.align	2
	.globl	createThread
	.align	2
	.globl	waitThread

       	.align	2
       	.globl	activeContext_p
	.align	2
	.globl	suspendContext
	.align	2
	.globl	resumeContext

       	.align	2
	.globl	writePort
	.align	2
	.globl	readPort
#	.align	2
#	.globl	testPort


        .align	2
        .globl	swapThreadOut
        .align	2
        .globl	swapThreadIn

        
        .align	2
        .globl	_startThread
        .align	2
        .globl	_swapContextOut
        .align	2
        .globl	_swapContextIn
        
	.text



#
# s e t M M U c o n f i g 
#
# This function set MMU configuration for a given context.
#
# Parameters
#   $4 context
#   $5 value
#                                
# WARING: There is no bound check !
#        

        .ent    setMMUconfig
setMMUconfig:
        addu    $4,$4,0xc0
        wport   $5,$4
        subu    $4,$4,0xc0
        jr      $31
        .end    setMMUconfig        

        



#
# g e t M M U c o n f i g 
#
# This function reads MMU configuration of a given context.
#
# Parameters
#   $4 context
#
# Returns MMU configuration value
#        
# WARING: There is no bound check !
#        

        .ent    getMMUconfig
getMMUconfig:
        addu    $4,$4,0xc0
        rport   $2,$4
        subu    $4,$4,0xc0
        jr      $31
        .end    getMMUconfig        

        


#
# _ s t a r t T h r e a d
#
# This function is called by a new created thread.
#
     
_startThread:
        .ent    _startThread
        li      $sp,0x1000
        rport   $sp,$sp                 
#        move    $fp,$sp
        lw      $4,4($sp)               # threadID
        lw      $5,8($sp)               # argv
        lw      $3,12($sp)              # function to execute
        lw      $28,16($sp)             # gp register
        subu    $sp,$sp,16
        jalr    $31,$3
_sT1:   lw      $3,16($sp)              # termination port
        wport   $2,$3                   # function return
        wport   $0,$0                   # endless wait
        .end    _startThread
        
                                
                
#        
# c r e a t e T h r e a d
# 
# Parameters
#   $4 threadId
#   $5 context
#   $6 (stack)
#   $7 (function to execute)
#      (function parameter)
#
# context + 0x1000 is returned
# 

       	.ent	createThread
createThread:
        subu    $6,$6,32
        addu    $2,$4,0x1000
        sw      $2,($6)                 # port used to terminate
        sw      $4,4($6)                # threadID
        lw      $2,16($fp)              # fifth parameter
        sw      $2,8($6)                # argv
        sw      $7,12($6)               # function to execute   
        sw      $28,16($6)              # r28 will be used by the child
        la      $2,_startThread
        addu    $5,$5,0x80              # port to write PC
        wport   $2,$5                   # writing new PC
        subu    $5,$5,0x80
        li      $2,0x1000
        wport   $6,$2                   # transmiting thread's stack pointer
        addu    $6,$6,32
	j	$31
	.end	createThread


        
#        
# w a i t T h r e a d
#
# Parameters
#   $4 threadId
#
# return value returned by the context
#        
                        
	.ent	waitThread
waitThread:
        addu    $2,$4,0x1000
        rport   $2,$2                   # return value
      	j	$31
	.end	waitThread



       


#        
# a c t i v e C o n t e x t _ p
#
# tests if the given context is running
#                      
# parameter:    
#   $4 context number
#
# returns 0 if the context is not running
#
        
        .ent    activeContext_p
activeContext_p:
	subu	$sp,$sp,32
	sw	$31,8($sp)
	sw	$fp,4($sp)
	move	$fp,$sp
        sw      $5,12($fp)
        sw      $6,16($fp)
        rport   $2,$0
        li      $6,0x1
        sll     $5,$6,$4
        and     $2,$2,$5
        lw      $5,12($fp)
        lw      $6,16($fp)
	move	$sp,$fp			# sp not trusted here
	lw	$31,8($sp)
	lw	$fp,4($sp)
	addu	$sp,$sp,32
	j	$31
        .end    activeContext_p



        

# s u s p e n d C o n t e x t
#
# stop execution of the given context.
#
# parameters:   
#   $4 context to stop
#
# return value: 
#   $2 PC of the suspended context
#                        
       	.ent	suspendContext
suspendContext:
        addu    $2,$4,0x80
        rport   $2,$2
	j	$31
	.end	suspendContext



        
#        
# r e s u m e C o n t e x t
#
# resume execution of the given context.
#
# parameters:   
#   $4 context to resume
#
# return value: 
#   $2 PC of the resumed context
#                        
       	.ent	resumeContext
resumeContext:
        addu    $4,$4,0x80
        rport   $2,$4
        wport   $2,$4
        subu    $4,$4,0x80
	j	$31
	.end	resumeContext



        

# w r i t e P o r t 
#
# send a word data to a given port.
#
# parameters:
#   $4 data to send
#   $5 port number                
#
        .ent writePort
writePort:  
        wport   $4,$5
	j	$31
        .end writePort


        
#        
# r e a d P o r t
#
# receive a word data from a given port.
#
# parameters:
#   $4 (data read)
#   $5 port number
#
        .ent readPort
readPort:  
	subu	$sp,$sp,32
	sw	$31,8($sp)
	sw	$fp,4($sp)
	move	$fp,$sp
        sw      $6,16($fp)
        rport   $6,$5
        sw      $6,($4)
        lw      $6,16($fp)
	move	$sp,$fp			# sp not trusted here
	lw	$31,8($sp)
	lw	$fp,4($sp)
	addu	$sp,$sp,32
	j	$31
        .end readPort




#
# _ c o n t e x t O u t
#
# This procedure is called by swapThreadOut
#        

        .ent    _contextOut
_contextOut:                     
        .set noat       
#        subu    $sp,$sp,128
        sw     $0,-256($sp)
        sw     $1,-252($sp)
        sw     $2,-248($sp)
        sw     $3,-244($sp)
        sw     $4,-240($sp)
        sw     $5,-236($sp)
        sw     $6,-232($sp)
        sw     $7,-228($sp)
        sw     $8,-224($sp)
        sw     $9,-220($sp)
        sw     $10,-216($sp)
        sw     $11,-212($sp)
        sw     $12,-208($sp)
        sw     $13,-204($sp)
        sw     $14,-200($sp)
        sw     $15,-196($sp)
        sw     $16,-192($sp)
        sw     $17,-188($sp)
        sw     $18,-184($sp)
        sw     $19,-180($sp)
        sw     $20,-176($sp)
        sw     $21,-172($sp)
        sw     $22,-168($sp)
        sw     $23,-164($sp)
        sw     $24,-160($sp)
        sw     $25,-156($sp)
        sw     $26,-152($sp)
        sw     $27,-148($sp)
        sw     $28,-144($sp)
        sw     $30,-136($sp)
        sw     $31,-132($sp)
        mflo    $6
        mfhi    $7
        dsw     $6,-264($sp)
        add     $6,$0,$0
        bc1f    cO_1
        li      $6,1
cO_1:   sw      $6,-268($sp)
        s.d     $f0,-128($sp)
        s.d     $f2,-120($sp)
        s.d     $f4,-112($sp)
        s.d     $f6,-104($sp)
        s.d     $f8,-96($sp)
        s.d     $f10,-88($sp)
        s.d     $f12,-80($sp)
        s.d     $f14,-72($sp)
        s.d     $f16,-64($sp)
        s.d     $f18,-56($sp)
        s.d     $f20,-48($sp)
        s.d     $f22,-40($sp)
        s.d     $f24,-32($sp)
        s.d     $f26,-24($sp)
        s.d     $f28,-16($sp)
        s.d     $f30,-8($sp)
        li      $4,0x2000       # swap port
        wport   $sp,$4
        wport   $0,$0           # context stopped
        .end    _contextOut


        
#
# s w a p T h r e a d O u t
#
# Makes a context to dump all its regsiters on its stack and stop.
#
# parameters:   
#    $4 context
#    $5 (sp)
#    $6 (pc)        
#
        .ent swapThreadOut
swapThreadOut:
        addu    $4,$4,0x80              # port to read PC
        rport   $2,$4                   # reading PC/stopping
        sw      $2,0($6)                # store PC value
        la      $2,_contextOut
        wport   $2,$4                   # swap out routine
        subu    $4,$4,0x80
        li      $2,0x2000
        rport   $2,$2                   # reading thread's stack pointer
        sw      $2,0($5)
	j	$31
        .end swapThreadOut

                




#
# _ c o n t e x t I n
#
# This procedure is called by swapThreadIn
#        

        .ent    _contextIn
        .set noat
_contextIn:                     
        li      $4,0x2001       # swapin port
        rport   $sp,$4
#        addu    $sp,$sp,128
        dlw     $6,-264($sp)
        mtlo    $6
        mthi    $7
        lw      $6,-268($sp)
        mtc1    $0,$f0
        mtc1    $6,$f2
        c.eq.s  $f0,$f2
        l.d     $f0,-128($sp)
        l.d     $f2,-120($sp)
        l.d     $f4,-112($sp)
        l.d     $f6,-104($sp)
        l.d     $f8,-96($sp)
        l.d     $f10,-88($sp)
        l.d     $f12,-80($sp)
        l.d     $f14,-72($sp)
        l.d     $f16,-64($sp)
        l.d     $f18,-56($sp)
        l.d     $f20,-48($sp)
        l.d     $f22,-40($sp)
        l.d     $f24,-32($sp)
        l.d     $f26,-24($sp)
        l.d     $f28,-16($sp)
        l.d     $f30,-8($sp)
        lw      $0,-256($sp)
        lw      $1,-252($sp)
        lw      $2,-248($sp)
        lw      $3,-244($sp)
        lw      $4,-240($sp)
        lw      $5,-236($sp)
        lw      $6,-232($sp)
        lw      $7,-228($sp)
        lw      $8,-224($sp)
        lw      $9,-220($sp)
        lw      $10,-216($sp)
        lw      $11,-212($sp)
        lw      $12,-208($sp)
        lw      $13,-204($sp)
        lw      $14,-200($sp)
        lw      $15,-196($sp)
        lw      $16,-192($sp)
        lw      $17,-188($sp)
        lw      $18,-184($sp)
        lw      $19,-180($sp)
        lw      $20,-176($sp)
        lw      $21,-172($sp)
        lw      $22,-168($sp)
        lw      $23,-164($sp)
        lw      $24,-160($sp)
        lw      $25,-156($sp)
        lw      $26,-152($sp)
        lw      $27,-148($sp)
        lw      $28,-144($sp)
        li      $31,0x2001
        lw      $30,-136($sp)
        wport   $sp,$31         # send message to creator
        lw      $31,-132($sp)
        wport   $0,$0           # context stopped
        .end    _contextIn
                
        
#
# s w a p T h r e a d I n
#
# Makes a context to load thread.
#
# parameters:   
#    $4 context
#    $5 sp
#    $6 pc        
#

        
        .ent swapThreadIn
swapThreadIn:  
	subu	$sp,$sp,32
	sw	$8,8($sp)
	sw	$9,12($sp)
        addu    $4,$4,0x80              # port to write PC
        la      $2,_contextIn
        wport   $2,$4                   # swap in routine
        li      $2,0x2001
        wport   $5,$2                   # sending sp
        rport   $2,$2                   # waiting for semicompletion...
_sTI:   rport   $2,$0   
        li      $9,0x1
        sll     $8,$9,$4
        and     $2,$2,$8
        bne     $0,$2,_sTI                
        wport   $6,$4                   # here the context is stopped
        subu    $4,$4,0x80
	lw	$8,8($sp)
	lw	$9,12($sp)
	addu	$sp,$sp,32
	j	$31
        .end swapThreadIn




